@@ -357,7 +357,7 @@ class OntologyProperty(OntologyObject):
     def __init__(self, kind: PropertyType, tenant_id: str, context: str, name: OntologyPropertyReference,
                  icon: str = None,
                  property_domain: Optional[OntologyClassReference] = None,
-                 property_range: Optional[OntologyClassReference] = None,
+                 property_range: Optional[Union[OntologyClassReference, DataPropertyType]] = None,
                  labels: Optional[List[Label]] = None,
                  comments: Optional[List[Comment]] = None,
                  subproperty_of: Optional[OntologyPropertyReference] = None,
@@ -366,7 +366,7 @@ class OntologyProperty(OntologyObject):
         self.__subproperty_of: OntologyPropertyReference = subproperty_of
         self.__inverse_property_of: OntologyPropertyReference = inverse_property_of
         self.__domain: OntologyClassReference = property_domain
-        self.__range: OntologyClassReference = property_range
+        self.__range: Optional[Union[OntologyClassReference, DataPropertyType]] = property_range
         self.__reference: OntologyPropertyReference = name
         super().__init__(tenant_id, name.iri, icon, labels, comments, context)
 
@@ -400,7 +400,7 @@ class OntologyProperty(OntologyObject):
         return self.__domain
 
     @property
-    def range(self) -> OntologyClassReference:
+    def range(self) -> Optional[Union[OntologyClassReference, DataPropertyType]]:
         return self.__range
 
     def __repr__(self):
@@ -598,13 +598,12 @@ class Ontology(object):
     Ontology
     --------
     The ontology consists of classes and properties.
-
     """
 
     def __init__(self):
-        self.__classes: Dict[OntologyClassReference, OntologyClass] = {}
-        self.__data_properties: Dict[OntologyPropertyReference, OntologyProperty] = {}
-        self.__object_properties: Dict[OntologyPropertyReference, OntologyProperty] = {}
+        self.__classes: Dict[str, OntologyClass] = {}
+        self.__data_properties: Dict[str, OntologyProperty] = {}
+        self.__object_properties: Dict[str, OntologyProperty] = {}
 
     def add_class(self, class_obj: OntologyClass):
         """
@@ -615,7 +614,7 @@ class Ontology(object):
         class_obj: OntologyClass
             Class object
         """
-        self.__classes[class_obj.reference] = class_obj
+        self.__classes[class_obj.reference.iri] = class_obj
 
     def add_properties(self, prop_obj: OntologyProperty):
         """
@@ -626,9 +625,9 @@ class Ontology(object):
         prop_obj: OntologyProperty
         """
         if prop_obj.is_data_property:
-            self.__data_properties[prop_obj.reference] = prop_obj
+            self.__data_properties[prop_obj.reference.iri] = prop_obj
         else:
-            self.__object_properties[prop_obj.reference] = prop_obj
+            self.__object_properties[prop_obj.reference.iri] = prop_obj
 
     @property
     def data_properties(self) -> List[OntologyProperty]:
@@ -645,6 +644,14 @@ class Ontology(object):
         """All classes."""
         return list(self.__classes.values())
 
+    def __check_hierarchy__(self, clz: OntologyClassReference, domain: OntologyClassReference) -> bool:
+        current_clz: OntologyClass = self.get_class(clz)
+        while current_clz is not None:
+            if current_clz.reference == domain:
+                return True
+            current_clz = self.get_class(current_clz.subclass_of)
+        return False
+
     def get_class(self, class_reference: OntologyClassReference) -> Optional[OntologyClass]:
         """
         Get class instance by reference.
@@ -659,7 +666,7 @@ class Ontology(object):
         instance: Optional[OntologyClass]
             Instance of ontology class.
         """
-        return self.__classes.get(class_reference)
+        return self.__classes.get(class_reference.iri)
 
     def get_object_properties(self, property_reference: OntologyPropertyReference) -> Optional[OntologyProperty]:
         """
@@ -675,7 +682,7 @@ class Ontology(object):
         instance: Optional[OntologyProperty]
             Instance of ontology object property.
         """
-        return self.__object_properties.get(property_reference)
+        return self.__object_properties.get(property_reference.iri)
 
     def get_data_properties(self, property_reference: OntologyPropertyReference) -> Optional[OntologyProperty]:
         """
@@ -691,7 +698,54 @@ class Ontology(object):
         instance: Optional[OntologyProperty]
             Instance of ontology object property.
         """
-        return self.__data_properties.get(property_reference)
+        return self.__data_properties.get(property_reference.iri)
+
+    def data_properties_for(self, cls_reference: OntologyClassReference) -> List[OntologyPropertyReference]:
+        """
+        Retrieve a list of data properties.
+
+        Parameters
+        ----------
+        cls_reference: OntologyClassReference
+            Class of the ontology
+
+        Returns
+        -------
+        data_properties: List[OntologyPropertyReference]
+            List of data properties, where domain fit for the class of one of its super classes.
+        """
+        data_properties: List[OntologyPropertyReference] = []
+        clz: Optional[OntologyClass] = self.get_class(cls_reference)
+        if clz is not None:
+            for dp in self.data_properties:
+                if self.__check_hierarchy__(clz.reference, dp.domain):
+                    data_properties.append(dp.reference)
+        return data_properties
+
+    def object_properties_for(self, cls_reference: OntologyClassReference) -> List[OntologyPropertyReference]:
+        """
+        Retrieve a list of object properties.
+
+        Parameters
+        ----------
+        cls_reference: OntologyClassReference
+            Class of the ontology
+
+        Returns
+        -------
+        object_properties: List[OntologyPropertyReference]
+            List of object properties, where domain fit for the class of one of its super classes.
+        """
+        object_properties: List[OntologyPropertyReference] = []
+        clz: Optional[OntologyClass] = self.get_class(cls_reference)
+        if clz is not None:
+            for dp in self.object_properties:
+                if self.__check_hierarchy__(clz.reference, dp.domain):
+                    object_properties.append(dp.reference)
+        return object_properties
+
+    def __repr__(self):
+        return f'<Ontology> : classes:= {self.classes}'
 
 
 class ThingObject(abc.ABC):
@@ -774,6 +828,49 @@ class ThingObject(abc.ABC):
     def label(self, value: List[Label]):
         self.__label = value
 
+    def add_label(self, label: str, language_code: LanguageCode):
+        """Adding an label for entity.
+
+        Parameters
+        ----------
+        label: str
+            Label
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        """
+        self.__label.append(Label(label, language_code, True))
+
+    def update_label(self, value: str, language_code: LanguageCode):
+        """Update or creates a label for a specific language.
+
+        Parameters
+        ----------
+        value: str
+            Value to be set
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        """
+        for label in self.label:
+            if label.language_code == language_code:
+                label.content = value
+                return
+        # Label with language does not exist, so create a new label
+        self.add_label(value, language_code)
+
+    def remove_label(self, language_code: LanguageCode):
+        """
+        Remove label for entity if it exists for language.
+
+        Parameters
+        ----------
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        """
+        for idx in range(len(self.label)):
+            if self.label[idx].language_code == language_code:
+                del self.__label[idx]
+                break
+
     def label_lang(self, language_code: LanguageCode) -> Optional[Label]:
         """
         Get label for language_code code.
@@ -884,6 +981,35 @@ class ThingObject(abc.ABC):
     def description(self, value: List[Description]):
         self.__description = value
 
+    def add_description(self, description: str, language_code: LanguageCode):
+        """Adding an description for entity.
+
+        Parameters
+        ----------
+        description: str
+            Description
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        """
+        self.__description.append(Description(description=description, language_code=language_code))
+
+    def update_description(self, value: str, language_code: LanguageCode):
+        """Update or creates a description for a specific language.
+
+        Parameters
+        ----------
+        value: str
+            Value to be set
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        """
+        for desc in self.description:
+            if desc.language_code == language_code:
+                desc.content = value
+                return
+        # Description with language does not exist, so create a new description
+        self.add_description(value, language_code)
+
     def description_lang(self, language_code: str) -> Optional[Description]:
         """
         Get description for entity.
@@ -897,11 +1023,25 @@ class ThingObject(abc.ABC):
         label: LocalizedContent
             Returns the  label for a specific language_code code
         """
-        for label in self.description:
-            if label.language_code == language_code:
-                return label
+        for desc in self.description:
+            if desc.language_code == language_code:
+                return desc
         return None
 
+    def remove_description(self, language_code: LanguageCode):
+        """
+        Remove description for entity if it exists for language.
+
+        Parameters
+        ----------
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        """
+        for idx in range(len(self.description)):
+            if self.description[idx].language_code == language_code:
+                del self.__description[idx]
+                break
+
     @property
     def concept_type(self) -> OntologyClassReference:
         """Concept type."""
@@ -948,6 +1088,42 @@ class ThingObject(abc.ABC):
     def alias(self, alias: List[Label]):
         self.__alias = alias
 
+    def alias_lang(self, language_code: LanguageCode) -> List[Label]:
+        """
+        Get alias for language_code code.
+
+        Parameters
+        ----------
+        language_code: LanguageCode
+            Requested language_code code
+        Returns
+        -------
+        aliases: List[Label]
+            Returns a list of aliases for a specific language code
+        """
+        aliases: List[Label] = []
+        for a in self.alias:
+            if a.language_code == language_code:
+                aliases.append(a)
+        return aliases
+
+    def update_alias(self, value: str, language_code: LanguageCode):
+        """Update or creates a alias for a specific language.
+
+        Parameters
+        ----------
+        value: str
+            Value to be set
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        """
+        for a in self.alias:
+            if a.language_code == language_code:
+                a.content = value
+                return
+        # Label with language does not exist, so create a new label
+        self.add_alias(value, language_code=language_code)
+
     def add_relation(self, prop: ObjectProperty):
         """Adding a relation to the entity.
 
@@ -1069,7 +1245,7 @@ class ThingObject(abc.ABC):
         return isinstance(other, ThingObject) and other.uri == self.uri
 
     def __repr__(self):
-        return f'<{self.concept_type}: uri:={self.uri}, labels:={self.label}, ' \
+        return f'<{self.concept_type.iri if self.__concept_type else "UNSET"}: uri:={self.uri}, labels:={self.label}, '\
                f'tenant access right:={self.tenant_access_right}]>'
 
 
