@@ -2,7 +2,7 @@
 # Copyright © 2021 Wacom. All rights reserved.
 import enum
 import urllib.parse
-from typing import List, Dict, Any, Tuple
+from typing import List, Dict, Any, Tuple, Optional
 
 import requests
 from requests import Response
@@ -551,9 +551,8 @@ class WacomKnowledgeService(WacomServiceAPIClient):
                     things[r[SUBJECT]].add_relation(ObjectProperty(relation, outgoing=[r[OBJECT]]))
             return things, relations
 
-    def listing(self, auth_key: str, filter_type: OntologyClassReference, page_id: str = 0, limit: int = 30,
-                language_code: LanguageCode = 'en_US') \
-            -> Tuple[List[ThingObject], int, str]:
+    def listing(self, auth_key: str, filter_type: OntologyClassReference, page_id: Optional[str] = None,
+                limit: int = 30, language_code: LanguageCode = 'en_US') -> Tuple[List[ThingObject], int, str]:
         """
         List all entities visible to users.
 
@@ -563,7 +562,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
             Auth key from user
         filter_type: OntologyClassReference
             Filtering with concept
-        page_id: str
+        page_id: Optional[str]
             Page id. Start from this page id
         limit: int
             Limit of the returned entities
@@ -691,6 +690,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         if response.ok:
             return WacomKnowledgeService.__search_results__(response.json())
         raise WacomServiceException(f'Search on labels {search_term} failed. '
+                                    f'{parameters}'
                                     f'Response code:={response.status_code}, exception:= {response.content}')
 
     def search_labels(self, auth_key: str, search_term: str, language_code: LanguageCode, limit: int = 30,
