@@ -8,8 +8,8 @@ import requests
 from requests import Response
 
 from knowledge.base.entity import OntologyContext, LocalizedContent, Label, Comment
-from knowledge.base.ontology import OntologyClassReference, OntologyPropertyReference, OntologyProperty, OntologyClass,\
-    PropertyType
+from knowledge.base.ontology import OntologyClassReference, OntologyPropertyReference, OntologyProperty, OntologyClass, \
+    PropertyType, THING_CLASS, DataPropertyType
 from knowledge.services.base import WacomServiceAPIClient, WacomServiceException
 from knowledge.services.graph import AUTHORIZATION_HEADER_FLAG
 
@@ -217,7 +217,8 @@ class OntologyService(WacomServiceAPIClient):
             return OntologyProperty.from_dict(response.json())
         raise WacomServiceException(f'Response code:={response.status_code}, exception:= {response.text}')
 
-    def create_concept(self, auth_key: str, context: str, name: str, subclass_of: Optional[str],
+    def create_concept(self, auth_key: str, context: str, reference: OntologyClassReference,
+                       subclass_of: OntologyClassReference = THING_CLASS,
                        icon: Optional[str] = None, labels: Optional[List[Label]] = None,
                        comments: Optional[List[Comment]] = None) -> Dict[str, str]:
         """Create concept class.
@@ -231,9 +232,9 @@ class OntologyService(WacomServiceAPIClient):
             Auth key from user.
         context: str
             Context of ontology
-        name: str
+        reference: OntologyClassReference
             Name of the concept
-        subclass_of: Optional[str]
+        subclass_of: OntologyClassReference (default:=wacom:core#Thing)
             Super class of the concept
         icon: Optional[str] (default:= None)
             Icon representing the concept
@@ -255,8 +256,8 @@ class OntologyService(WacomServiceAPIClient):
             AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
         }
         payload: Dict[str, Any] = {
-            SUB_CLASS_OF_TAG: subclass_of,
-            NAME_TAG: name,
+            SUB_CLASS_OF_TAG: subclass_of.iri,
+            NAME_TAG: reference.iri,
             LABELS_TAG: [],
             COMMENTS_TAG: [],
             ICON_TAG: icon
@@ -332,13 +333,14 @@ class OntologyService(WacomServiceAPIClient):
         raise WacomServiceException(f'Update of concept failed. '
                                     f'Response code:={response.status_code}, exception:= {response.text}')
 
-    def create_property(self, auth_key: str, context: str, property_type: PropertyType, name: OntologyPropertyReference,
-                        domain_cls: OntologyClassReference, range_cls: OntologyClassReference,
-                        inverse_of: Optional[OntologyPropertyReference] = None,
-                        subproperty_of: Optional[OntologyPropertyReference] = None,
-                        icon: Optional[str] = None,
-                        labels: Optional[List[LocalizedContent]] = None,
-                        comments: Optional[List[LocalizedContent]] = None) -> Dict[str, str]:
+    def create_object_property(self, auth_key: str, context: str,
+                               reference: OntologyPropertyReference,
+                               domain_cls: OntologyClassReference, range_cls: OntologyClassReference,
+                               inverse_of: Optional[OntologyPropertyReference] = None,
+                               subproperty_of: Optional[OntologyPropertyReference] = None,
+                               icon: Optional[str] = None,
+                               labels: Optional[List[LocalizedContent]] = None,
+                               comments: Optional[List[LocalizedContent]] = None) -> Dict[str, str]:
         """Create property.
 
         **Remark:**
@@ -350,9 +352,7 @@ class OntologyService(WacomServiceAPIClient):
             Auth key from user.
         context: str
             Context of ontology
-        property_type: PropertyType
-            Type of the property
-        name: OntologyPropertyReference
+        reference: OntologyPropertyReference
             Name of the concept
         domain_cls: OntologyClassReference
             IRI of the domain
@@ -383,12 +383,12 @@ class OntologyService(WacomServiceAPIClient):
             AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
         }
         payload: Dict[str, Any] = {
-            KIND_TAG: property_type.value,
+            KIND_TAG: PropertyType.OBJECT_PROPERTY.value,
             DOMAIN_TAG: domain_cls.iri,
             RANGE_TAG: range_cls.iri,
-            SUB_PROPERTY_OF_TAG: subproperty_of.iri,
-            INVERSE_OF_TAG: inverse_of.iri,
-            NAME_TAG: name.iri,
+            SUB_PROPERTY_OF_TAG: subproperty_of.iri if subproperty_of is not None else None,
+            INVERSE_OF_TAG: inverse_of.iri if inverse_of is not None else None,
+            NAME_TAG: reference.iri,
             LABELS_TAG: [],
             COMMENTS_TAG: [],
             ICON_TAG: icon
@@ -402,7 +402,75 @@ class OntologyService(WacomServiceAPIClient):
         response: Response = requests.post(url, headers=headers, json=payload, verify=self.verify_calls)
         if response.ok:
             return response.json()
-        raise WacomServiceException(f'Update of concept failed. '
+        raise WacomServiceException(f'Creation of object property failed. '
+                                    f'Response code:={response.status_code}, exception:= {response.text}')
+
+    def create_data_property(self, auth_key: str, context: str,
+                             reference: OntologyPropertyReference,
+                             domain_cls: OntologyClassReference, range_cls: DataPropertyType,
+                             subproperty_of: Optional[OntologyPropertyReference] = None,
+                             icon: Optional[str] = None,
+                             labels: Optional[List[LocalizedContent]] = None,
+                             comments: Optional[List[LocalizedContent]] = None) -> Dict[str, str]:
+        """Create data property.
+
+        **Remark:**
+        Only works for users with role 'TenantAdmin'.
+
+        Parameters
+        ----------
+        auth_key: str
+            Auth key from user.
+        context: str
+            Context of ontology
+        reference: OntologyPropertyReference
+            Name of the concept
+        domain_cls: OntologyClassReference
+            IRI of the domain
+        range_cls: DataPropertyType
+            Data property type
+        subproperty_of: Optional[OntologyPropertyReference] = None,
+            Super property of the concept
+        icon: Optional[str] (default:= None)
+            Icon representing the concept
+        labels: Optional[List[Label]] (default:= None)
+            Labels for the class
+        comments: Optional[List[Comment]] (default:= None)
+            Comments for the class
+
+        Returns
+        -------
+        result: Dict[str, str]
+            Result from the service
+
+        Raises
+        ------
+        WacomServiceException
+            If the ontology service returns an error code, exception is thrown.
+        """
+        headers: Dict[str, str] = {
+            AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
+        }
+        payload: Dict[str, Any] = {
+            KIND_TAG: PropertyType.DATA_PROPERTY.value,
+            DOMAIN_TAG: domain_cls.iri,
+            RANGE_TAG: range_cls.value,
+            SUB_PROPERTY_OF_TAG: subproperty_of.iri if subproperty_of is not None else None,
+            NAME_TAG: reference.iri,
+            LABELS_TAG: [],
+            COMMENTS_TAG: [],
+            ICON_TAG: icon
+        }
+        context_url: str = urllib.parse.quote_plus(context)
+        for label in labels if labels is not None else []:
+            payload[LABELS_TAG].append({TEXT_TAG: label.content, LANGUAGE_CODE: label.language_code})
+        for comment in comments if comments is not None else []:
+            payload[COMMENTS_TAG].append({TEXT_TAG: comment.content, LANGUAGE_CODE: comment.language_code})
+        url: str = f'{self.service_base_url}{OntologyService.PROPERTIES_ENDPOINT.format(context_url)}'
+        response: Response = requests.post(url, headers=headers, json=payload, verify=self.verify_calls)
+        if response.ok:
+            return response.json()
+        raise WacomServiceException(f'Creation of data property failed. '
                                     f'Response code:={response.status_code}, exception:= {response.text}')
 
     def create_context(self, auth_key: str, name: str, base_uri: Optional[str] = None, icon: Optional[str] = None,
